@@ -1,4 +1,4 @@
-# Copyright (C) 1998 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,18 +25,17 @@ import sys
 import string
 import re
 import traceback
-import Message
-import Errors
-import mm_cfg
-import Utils
-import MailList
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+from Mailman import Message
+from Mailman import Errors
+from Mailman import mm_cfg
+from Mailman import Utils
+from Mailman.Handlers import HandlerAPI
+from Mailman.Logging.Syslog import syslog
+from Mailman.pythonlib.StringIO import StringIO
 
 
+
 MAXERRORS = 5
 MAXCOLUMN = 70
 
@@ -66,7 +65,7 @@ have a mail reader that supports MIME.""",
 option_info = {'digest'  : 0,
                'nomail'  : mm_cfg.DisableDelivery,
                'notmetoo': mm_cfg.DontReceiveOwnPosts,
-               'ack'     : mm_cfg.AcknowlegePosts,
+               'ack'     : mm_cfg.AcknowledgePosts,
                'plain'   : mm_cfg.DisableMime,
                'hide'    : mm_cfg.ConcealSubscription
                }
@@ -110,28 +109,38 @@ class MailCommandHandler:
         self.__errors = self.__errors + 1
         self.AddToResponse(text, trunc=trunc, prefix=prefix)
 	
-    def ParseMailCommands(self):
-	msg = Message.IncomingMessage()
+    def ParseMailCommands(self, msg):
+        # Break any infloops.  If this has come from a Mailman server then
+        # it'll have this header.  It's still possible to infloop between two
+        # servers because there's no guaranteed way to know it came from a
+        # bot.
+        if msg.get('x-beenthere') or msg.get('list-id'):
+            return
+        # check the autoresponse stuff
+        if self.autorespond_requests:
+            # TBD: this is a hack and is not safe with respect to errors in
+            # the Replybot module.  It should be redesigned to work with the
+            # robust delivery scheme.
+            from Mailman.Handlers import Replybot
+            Replybot.process(self, msg, msgdata={'torequest':1})
+            if self.autorespond_requests == 1:
+                # Yes, auto-respond and discard
+                return
 	subject = msg.getheader("subject")
         sender = string.lower(msg.GetSender())
         sender = string.split(sender, "@")[0]
         #
         # XXX: why 'orphanage'?
-        if sender in ['daemon', 'nobody', 'mailer-daemon', 'postmaster',
-                      'orphanage', 'postoffice']:
-            #
+        if sender in mm_cfg.LIKELY_BOUNCE_SENDERS:
             # This is for what are probably delivery-failure notices of
             # subscription confirmations that are, of necessity, bounced
             # back to the -request address.
-            self.LogMsg("bounce",
-                        ("%s: Mailcmd rejected"
-                         "\n\tReason: Probable bounced subscribe-confirmation"
-                         "\n\tFrom: %s"
-                         "\n\tSubject: %s"
-                         ),
-                        self._internal_name,
-                        msg.getheader('from'),
-                        subject)
+            syslog("bounce", "%s: Mailcmd rejected"
+                   "\n\tReason: Probable bounced subscribe-confirmation"
+                   "\n\tFrom: %s"
+                   "\n\tSubject: %s" % (self.internal_name(),
+                                        msg.getheader('from'),
+                                        subject))
             return
 	if subject:
 	    subject = string.strip(subject)
@@ -221,58 +230,59 @@ class MailCommandHandler:
 An unexpected Mailman error has occurred.
 
 Please forward your request to the human list administrator in charge of this
-list at <%s>.  The traceback is attached below and will be forwarded to the list
-administrator automatically.''' % admin)
+list at <%s>.  The traceback is attached below and will be forwarded to the
+list administrator automatically.''' % admin)
                         self.AddError(errmsg, trunc=0)
                         self.AddToResponse('\n' + tbmsg, trunc=0)
                         # log it to the error file
-                        self.LogMsg('error',
-                                    'Unexpected Mailman error:\n%s' % tbmsg)
+                        syslog('error',
+                               'Unexpected Mailman error:\n%s' % tbmsg)
                         # and send the traceback to the user
-                        self.SendTextToUser(subject='Unexpected Mailman error',
-                                            recipient=admin,
-                                            text='''\
+                        responsemsg = Message.UserNotification(
+                            admin, admin, 'Unexpected Mailman error',
+                            '''\
 An unexpected Mailman error has occurred in
 MailCommandHandler.ParseMailCommands().  Here is the traceback:
 
-''' + tbmsg,
-                                            add_headers=['Errors-To: %s' %
-                                                         admin,
-                                                         'X-No-Archive: yes',
-                                                         'Precedence: bulk'])
+''' + tbmsg)
+                        responsemsg['X-No-Archive'] = 'yes'
+                        # TBD: This should check the return value
+                        HandlerAPI.DeliverToUser(self, responsemsg)
                         break
         # send the response
         if not self.__NoMailCmdResponse:
+            adminaddr = self.GetAdminEmail()
+            requestaddr = self.GetRequestEmail()
             if self.__errors > 0:
                 header = Utils.wrap('''This is an automated response.
 
-There were problems with the email commands you sent to Mailman via
-the administrative address <%(sender)s>.
+There were problems with the email commands you sent to Mailman via the
+administrative address <%(sender)s>.
 
 To obtain instructions on valid Mailman email commands, send email to
-<%(sender)s> with the word "help" in the subject line or in the body
-of the message.
+<%(sender)s> with the word "help" in the subject line or in the body of the
+message.
 
-If you want to reach the human being that manages this mailing list,
-please send your message to <%(admin)s>.
+If you want to reach the human being that manages this mailing list, please
+send your message to <%(admin)s>.
 
 The following is a detailed description of the problems.
 
-''' % {'sender': self.GetRequestEmail(),
-       'admin' : self.GetAdminEmail(),
+''' % {'sender': requestaddr,
+       'admin' : adminaddr,
        })
                 self.__respbuf = header + self.__respbuf
-            self.SendMailCmdResponse(msg)
-
-    def SendMailCmdResponse(self, mail):
-        subject = 'Mailman results for %s' % self.real_name
-	self.SendTextToUser(subject   = subject,
-			    recipient = mail.GetSender(),
-			    sender    = self.GetRequestEmail(),
-			    text      = self.__respbuf)
-	self.__respbuf = ''
-        self.__errors = 0
-        self.__NoMailCmdResponse = 0
+            # send the response
+            subject = 'Mailman results for %s' % self.real_name
+            responsemsg = Message.UserNotification(msg.GetSender(),
+                                                   self.GetRequestEmail(),
+                                                   subject,
+                                                   self.__respbuf)
+            # TBD: This should check the return value
+            HandlerAPI.DeliverToUser(self, responsemsg)
+            self.__respbuf = ''
+            self.__errors = 0
+            self.__NoMailCmdResponse = 0
 
     def ProcessPasswordCmd(self, args, cmd, mail):
         if len(args) not in [0,2]:
@@ -295,7 +305,7 @@ The following is a detailed description of the problems.
             self.ConfirmUserPassword(sender, args[0])
 	    self.ChangeUserPassword(sender, args[1], args[1])
 	    self.AddToResponse('Succeeded.')
-	except Errors.MMListNotReady:
+	except Errors.MMListNotReadyError:
 	    self.AddError("List is not functional.")
 	except Errors.MMNotAMemberError:
 	    self.AddError("%s isn't subscribed to this list." % sender,
@@ -374,7 +384,7 @@ The following is a detailed description of the problems.
 		self.AddError("List only accepts digest members.")
 	    except Errors.MMCantDigestError:
 		self.AddError("List doesn't accept digest members.")
-	    except Errors.MMListNotReady:
+	    except Errors.MMListNotReadyError:
 		self.AddError("List is not functional.")
 	    except Errors.MMNoSuchUserError:
 		self.AddError("%s is not subscribed to this list."
@@ -403,8 +413,10 @@ The following is a detailed description of the problems.
 		listob = self
 	    else:
                 try:
+                    from Mailman import MailList
                     listob = MailList.MailList(listname, lock=0)
-                except Errors.MMUnknownListError:
+                except Errors.MMListError, e:
+                    # TBD: better error reporting
                     continue
 	    # We can mention this list if you already know about it.
 	    if not listob.advertised and listob <> self: 
@@ -437,7 +449,7 @@ background and instructions for subscribing to and using it, visit:
     %(url)s
 
 ''' % {'listname': self.real_name,
-       'url'     : self.GetAbsoluteScriptURL('listinfo'),
+       'url'     : self.GetScriptURL('listinfo', absolute=1),
        })
         self.AddToResponse(msg, trunc=0)
 
@@ -503,13 +515,17 @@ background and instructions for subscribing to and using it, visit:
 	    self.ConfirmUserPassword(addr, args[0])
 	    self.DeleteMember(addr, "mailcmd")
 	    self.AddToResponse("Succeeded.")
-	except Errors.MMListNotReady:
+	except Errors.MMListNotReadyError:
 	    self.AddError("List is not functional.")
 	except (Errors.MMNoSuchUserError, Errors.MMNotAMemberError):
 	    self.AddError("%s is not subscribed to this list." % addr,
                           trunc=0)
 	except Errors.MMBadPasswordError:
 	    self.AddError("You gave the wrong password.")
+        except Errors.MMBadUserError:
+            self.AddError('Your stored password is bogus.')
+            syslog('subscribe', 'User %s on list %s has no password' %
+                   (addr, self.internal_name()))
 
     def ProcessSubscribeCmd(self, args, cmd, mail):
         """Parse subscription request and send confirmation request."""
@@ -557,16 +573,16 @@ background and instructions for subscribing to and using it, visit:
             # of the results of the mail command message
             #
             self.__NoMailCmdResponse = 1
-        except Errors.MMNeedApproval, admin_email:
+        except Errors.MMNeedApproval:
             self.AddToResponse(
-                "your subscription request has been forwarded the list "
-                "administrator\n"
-                "at %s for review." % admin_email, trunc=0)
+                "Your subscription request has been forwarded to the"
+                " list administrator\n"
+                "at %s for review." % self.GetAdminEmail(), trunc=0)
         except Errors.MMBadEmailError:
             self.AddError("Mailman won't accept the given email "
                           "address as a valid address.\n"
                           "(Does it have an @ in it???)")
-        except Errors.MMListNotReady:
+        except Errors.MMListNotReadyError:
             self.AddError("The list is not fully functional, and "
                           "can not accept subscription\n"
                           "requests.")
@@ -648,7 +664,7 @@ background and instructions for subscribing to and using it, visit:
             'help.txt',
             {'listname'    : self.real_name,
              'version'     : mm_cfg.VERSION,
-             'listinfo_url': self.GetAbsoluteScriptURL('listinfo'),
+             'listinfo_url': self.GetScriptURL('listinfo', absolute=1),
              'requestaddr' : self.GetRequestEmail(),
              'adminaddr'   : self.GetAdminEmail(),
              })
